import flask
from flask import Flask, render_template, g, abort, url_for, request

from beets.plugins import BeetsPlugin
from beets.ui import Subcommand
from beets.library import Transaction
import beets

import json

app = Flask(__name__)

# Utilities
def get_tracks():
    return app.config['lib'].items()

def get_albums():
    album_id = app.config['lib'].albums()[0].__getattr__('id')

    albums = []
    for a in app.config['lib'].albums():
        albums.append({"title":a.items().next().album, "id":a.id})
    return albums

def get_artists():
    # We have to build a list of unique artists and match them to album ids
    # Should be {artist : [artist_id, albumid1, albumid2]}

    artists = {}
    artists_count = 0

    for a in app.config['lib'].albums():
        album = a.items().next()
        artist = album.artist

        if artist not in artists:
            artists[artist] = [artists_count]
            artists_count += 1
        
        artists[artist].append(album.album_id)

    return artists

def _rep(obj):
    if isinstance(obj, beets.library.Item):
        out = dict(obj.record)
        del out['path']
        return out
    elif isinstance(obj, beets.library.Album):
        out = dict(obj._record)
        del out['artpath']
        out['items'] = [_rep(item) for item in obj.items()]
        return out

# Used in m3u code
def get_album_tracks(album_id):
    return [i.id for i in g.lib.get_album(album_id).items()]

@app.before_request
def before_request():
    g.lib = app.config['lib']

# End of m3u code stuffs

@app.route("/")
def index():
    return render_template("index.html", tracks = get_tracks(), albums = get_albums(), artists = get_artists())

@app.route('/album/<int:album_id>')
def single_album(album_id):
    return json.dumps(get_album_tracks(album_id))

@app.route('/item/<int:item_id>')
def single_item(item_id):
    item = g.lib.get_item(item_id)
    return flask.jsonify(_rep(item))

@app.route('/item/<int:item_id>/file')
def item_file(item_id):
    item = g.lib.get_item(item_id)
    return flask.send_file(item.path, mimetype = "audio/mpeg")

@app.route('/album/<int:album_id>/art')
def album_art(album_id):
    album = g.lib.get_album(album_id)
    if album.artpath:
        return flask.send_file(album.artpath)
    else:
        return ""

@app.route('/test')
def test():
    return render_template('test.html')

# What follows is the m3u code
@app.route('/m3u/track/<int:track_id>.m3u')
def m3u_track(track_id):
    return "http://" + request.host + '/' + url_for("item_file", item_id = track_id)

@app.route('/m3u/album/<int:album_id>.m3u')
def m3u_album(album_id):
    albums = get_album_tracks(album_id)
    return "\n".join(["http://" + request.host + '/' + url_for("item_file", item_id = track_id) for track_id in albums])

@app.route('/m3u/artist/<int:artist_id>.m3u')
def m3u_artist(artist_id):
    artists = get_artists()
    sub_list = []
    ret = []

    for a in artists:
        if artists[a][0] == artist_id:
            sub_list = artists[a][1:]
            break
    
    for album in sub_list:
        for track_id in get_album_tracks(album):
            ret.append("http://" + request.host + '/' + url_for("item_file", item_id = track_id))

    return "\n".join(ret)

class EdnaPlugin(BeetsPlugin):
    def commands(self):
        cmd = Subcommand('edna2', help='start a Web interface')
        cmd.parser.add_option('-d', '--debug', action="store_true",
                default=True, help='debug mode')

        def func(lib, config, opts, args):
            # Taken from beets source
            if args:
                host = args.pop(0)
            else:
                host = beets.ui.config_val(config, 'web', 'host', "127.0.0.1")

            if args:
                port = args.pop(0)
            else:
                port = beets.ui.config_val(config, 'web', 'port', str(5000))
            port = int(port)

            app.config['lib'] = lib
            app.run(host=host, port=port, debug = opts.debug)

        cmd.func = func
        return [cmd]
    
